Introduction      {#einleitung}
=====

\~German
Das ISEAFramework ist eine am Institut für Stromrichtertechnik und Elektrische Antriebe (ISEA) entwickelte Programmbibliothek zur generischen Simulation von Batterienetzwerken.
Basierend auf den charakteristischen Messergebnissen der Impedanzspektroskopie können beispielsweise Lithium-Ionen-Batterien oder Supercaps in der Simulation durch ein elektrisches Ersatzschaltbild angenähert werden.
Es ist möglich, ein solches Ersatzschaltbild durch eine Reihenschaltung einfacher Grundelemente zu approximieren.
Diese Grundelemente sind als Klassen im Framework hinterlegt und können in Reihen- und Parallelschaltungen zu einem Netzwerk verbunden werden.
Auf diese Weise können ganze Batteriepakete nachgebildet werden.
<br/><br/>
Zusätzlich kann ein thermisches Modell erstellt werden, welches es ermöglicht, die Temperaturverteilung und -Entwicklung in der Batterie bzw. im Batteriepack zu simulieren.
Dieses Modell kann gekoppelt mit dem elektrischen Modell laufen, so dass der Einfluss der Temperatur auf die elektrische Parametrierung simuliert werden kann.
<br/><br/>

Ablauf der Simulation
----
![Schematische Darstellung eines Simulationszyklus](Kreislauf.png)

Die Simulation läßt sich in drei Schritte unterteilen.


Initialisierungsphase:
----
- Aufstellen des elektrischen Ersatzschaltbildes
- Erstellen des thermischen Modells

Simulationszyklus:
-----
- 2. Setzen des geforderten Stroms
- 3. Update der Lookups
- 4. Setzen der algebraischen und differentiellen Gleichungen
- 5. Berechnung und Lösung des Statespaces
- 6. Berechnung der Ströme, Spannungen und thermischen Verlustleistungen
- 7. Übergabe der thermischen Verlustleistungen an das thermische Modell (Nicht eingezeichnet im Bild)
- 8. Berechnen und Weiterleiten der neuen Temperaturen  (Nicht eingezeichnet im Bild)

Aufräumen:
----
- Freigabe des genutzten Speichers

Nach einer Simulation können die Ergebnisse visualisiert werden.


![Visualisierten Ergebnisse eines Packs](visualisierung2.png)

<!--

Introduction {#einleitung}
=====
-->
\~English


The ISEAFramework is a library for generic battery simulation and is developed at the Institute for Power Electronics and Electrical Drives (ISEA) .
Models are based on datasets gained through electrochemical impedance spectroscopy (EIS) and a variety of different storage technologies can be described with these measurements,
e.g. lithium-ion batteries and double layer capacitors.
Results gained by EIS can be used to approximate the system through an electrical equivalent circuit and can be connected in different ways (parallel and serial).
This allows to describe storage systems in a generic way and to not only describe a single battery, but whole packs.

<br/><br/>
Additionally a thermal model can be simulated, which simulates temperature distributions and trends inside the battery and battery pack.
This model is coupled with the electrical model during runtime, so that the thermal heat, which is generated by the electrical model, is directly fed into the thermal simulation.
<br/><br/>

Simulation sequence
----
![Schematic of the simulation cycle](Kreislauf.png)


Initialization:
----
- Creation of the electrical model
- Creation of the thermal model

Simulation-cycle:
-----
- 2. Set requested current
- 3. Update of lookup tables
- 4. Calculation of differential and algebraic equations (DAE)
- 5. Solving DAE system and calculating state space vector
- 6. Calculating currents, voltages and thermal losses
- 7. Passing the thermal losses (head) to the thermal model (not depicted in the image)
- 8. Calculating new temperature distribution and closing the feedback loop through passing the temperature back to the electrical model (not depicted in the image)

Clean up:
----
- Release allocated memory

At the end of the simulation the results can be plotted.


![Visualized results of a battery pack simulation](visualisierung2.png)


